import numpy as np
import torch
import torch.optim as optim
from copy import deepcopy
from sample import Sample
from utils import get_ep_indices

'''
Define a external pareto class storing all computed policies on the current pareto front.
'''
class EP:
    def __init__(self, reference_point, num_select_solutions):
        self.obj_batch = np.array([])
        self.sample_batch = np.array([])
        self.selected_objs = []  # List to store selected objective values
        self.reference_point = reference_point
        self.num_select_solutions = num_select_solutions

    def index(self, indices, inplace=True):
        if inplace:
            self.obj_batch, self.sample_batch = \
                map(lambda batch: batch[np.array(indices, dtype=int)], [self.obj_batch, self.sample_batch])
        else:
            return map(lambda batch: deepcopy(batch[np.array(indices, dtype=int)]), [self.obj_batch, self.sample_batch])

    def update(self, sample_batch):
        self.sample_batch = np.append(self.sample_batch, np.array(deepcopy(sample_batch)))
        for sample in sample_batch:
            self.obj_batch = np.vstack([self.obj_batch, sample.objs]) if len(self.obj_batch) > 0 else np.array([sample.objs])

        if len(self.obj_batch) == 0: return
        print('index')
        ep_indices = get_ep_indices(self.obj_batch, self.reference_point)
        print('got index')
        self.index(ep_indices)
        print(self.obj_batch)
        self.filter_by_crowding_distance(self.num_select_solutions) #number of elite policies
        print(self.obj_batch)

    def calculate_crowding_distance(self, obj_batch):
        if len(obj_batch) == 0:
            return np.array([])
        
        num_samples = obj_batch.shape[0]
        num_objectives = obj_batch.shape[1]
        crowding_distances = np.zeros(num_samples)

        for i in range(num_objectives):
            obj_values = obj_batch[:, i]
            sorted_indices = np.argsort(obj_values)
            distances = np.zeros(num_samples)
            distances[1:-1] = (obj_values[sorted_indices[2:]] - obj_values[sorted_indices[:-2]]) / \
                              (obj_values[sorted_indices[-1]] - obj_values[sorted_indices[0]])
            
            # Set the boundary points to infinity or a large number to ensure they are selected
            distances[0] = distances[-1] = np.inf
            crowding_distances[sorted_indices] += distances

        return crowding_distances

    def filter_by_crowding_distance(self, N):
        crowding_distances = self.calculate_crowding_distance(self.obj_batch)
        sorted_indices = np.argsort(-crowding_distances)  # Get indices sorted by crowding distance

        # Select the top N policies, ensuring we don't select duplicates based on objective values
        new_indices = []
        for idx in sorted_indices:
            # Convert objectives to a tuple to use as a hashable type for comparison
            obj_tuple = tuple(self.obj_batch[idx])

            if obj_tuple not in self.selected_objs and len(new_indices) < N:
                new_indices.append(idx)
                self.selected_objs.append(obj_tuple)

        # Filter samples by these new indices using the existing self.index method
        self.index(new_indices)
        
    def random_selection(self, N):
        """Randomly select N policies without considering crowding distance."""
        if len(self.obj_batch) < N:
            N = len(self.obj_batch)
        
        random_indices = np.random.choice(len(self.obj_batch), size=N, replace=False)
        self.index(random_indices)
